/**
 *   File: ds2438.cpp
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#include "ds2438.h"
#include "nstring.h"

namespace CiematCMS {

//**********************************************************
//*********** CONSTRUCTORES Y DESTRUCTORES **************
//**********************************************************
DS2438::DS2438(I_OneWireMaster &master)
  : Dallas_1WireSlave(master) 
{
  DEFLOGINFO("Creamos un objeto DS2438");

  sensorResistance= 0;
  bit_IAD=true;
  bit_AD=true;

  // Al empezar, se supone que la configuracin del integrado no es buena.
  configChanged=true;

  this->clearMemPageData();
}

DS2438::~DS2438()
{
  DEFLOGINFO("Destruimos un objeto DS2438");
}

//**********************************************************
//*********** MTODOS PBLICOS **************
//**********************************************************
/** Devuelve los valores de temperatura, tensin y corriente medidos
 * por el dispositivo.
 * \fn DS2438::getTempVoltCurr(float *temp, float *volt, float *curr)
 */
void DS2438::getTempVoltCurr(float *temp, float *volt, float *curr) {

  DEFLOGINFO("DS2438: Entramos en getTempVoltCurr");

  // Invocamos la configuracin del integrado.
  configDevice();

  // Enviamos orden de conversion de temperatura y tension.
  master1W.reset1W();
  skipRom();
  master1W.write1WByte(CONVERT_T);

  DEFLOGDEBUG("DS2438: Enviado Conversin de T.");

  master1W.reset1W( );
  skipRom();
  master1W.write1WByte(CONVERT_V);

  DEFLOGDEBUG("DS2438: Enviado Conversin de V.");

  // Esperamos a que se produzca la conversin.
  usleep (WTIMECONVERT);

  // Leemos la pagina de memoria #0. Los datos se almacenan en el vector
  // auxiliar 
  readPage (0);

  DEFLOGDEBUG("DS2438: Leda pgina de memoria.");

  convert (temp, volt, curr);

  DEFLOGINFO("DS2438: Salimos de getTempVoltCurr");
}

/** Devuelve un entero long con el valor del tiempo transcurrido
 * (en segundos) desde que se activ el dispositivo.
 */
unsigned long DS2438::getElapsedTime(void){

  DEFLOGINFO("DS2438: Entramos en getElapsedTime");

  /* Limpiamos la pgina de trabajo */
  clearMemPageData();
  // Leemos la pgina 1, con la informacin de tiempo transcurrido.
  readPage (1);

  unsigned long time=0;

  time= paginaInterna.elapsedTime0 +
      (paginaInterna.elapsedTime1 * 0x100) +
      (paginaInterna.elapsedTime2 * 0x10000) +
      (paginaInterna.elapsedTime3 * 0x1000000);

  DEFLOGINFO("DS2438: Salimos de getElapsedTime");

  return time;
}

//**********************************************************
//*********** MTODOS GET Y SET **************
//**********************************************************
float DS2438::getSensorResistance() const {
  return sensorResistance;
}

void DS2438::setSensorResistance(float dato){
  sensorResistance= dato;
}

/** Activa la conversin A/D de la corriente.
 *  Permite as medir la corriente (default).
 */
void DS2438::enableCurrentAD(void){
  bit_IAD=true;
  configChanged=true;
}

/** Desactiva la conversin A/D de la corriente.
 */
void DS2438::disableCurrentAD(void){
  bit_IAD=false;
  configChanged=true;
}

/** Selecciona el pin VAD del integrado como entrada para la medida
 * de tensin.
 */
void DS2438::selectVADInput(void){
  bit_AD=false;
  configChanged=true;
}

/** Selecciona el pin VDD del integrado como entrada para la medida
 * de tensin (default).
 */
void DS2438::selectVDDInput(void){
  bit_AD=true;
  configChanged=true;
}

//**********************************************************
//*********** MTODOS PRIVADOS **************
//**********************************************************
/** Escribe todos los bits del registro de control a partir
 * de los atributos bit-* que se hayan definido en la clase.
 *
 * Este mtodo es til para ser llamado desde otros mtodos de acceso,
 * antes 
 * 
 */
void DS2438::configDevice(void){

  DEFLOGINFO("DS2438: Entramos en configDevice");

  if (configChanged) {
    DEFLOGINFO("DS2438: Procedemos a configurar");
    /* Preparamos con un 0 el registro de control/status
    y borramos el resto */
    clearMemPageData();

    // Activamos los bits del registro de control que convenga
    if (bit_IAD) paginaInterna.memPageData[0] |= CTRLBIT_IAD;
    if (bit_AD) paginaInterna.memPageData[0] |= CTRLBIT_AD;

    // Damos orden de escribir el primer byte del vector en la pgina 0
    writeToPage(0, 1);
  }

  configChanged=false;

  DEFLOGINFO("DS2438: Salimos de configDevice");
}

/*!
 * \fn DS2438::readPage (unsigned pageNum)
 * 
 * Lee una pagina de memoria del integrado y la vuelca en el
 * vector privado "memPageData".
 * El ltimo byte ledo es CRC.
 */
void DS2438::readPage (unsigned pageNum) {
  DEFLOGINFO("DS2438: Entramos en readPage");
  // Recall page #pageNum to scratchpad page #pageNum
  // Copiamos la pgina de memoria en una pgina de trabajo.
  master1W.reset1W();
  skipRom();
  master1W.write1WByte(RCLL_MEMPAGE);
  master1W.write1WByte(pageNum);

  DEFLOGDEBUG("DS2438: Enviado orden RCLL_MEMPAGE");

  // Read scratchpad page #pageNum
  // Indicamos que vamos a chupar los 8 bytes de la pgina
  // de memoria + CRC. Despus es obligatorio leerlos. Consultar datasheet.
  master1W.reset1W();
  skipRom();
  master1W.write1WByte(RD_SCRPAGE);
  master1W.write1WByte(pageNum);

  DEFLOGDEBUG("DS2438: Enviado orden RD_SCRPAGE");

  // Leemos los datos de la memPage
  for (int i=0; i< PAGELENGTH; i++)
    paginaInterna.memPageData[i] = master1W.read1WByte();

  DEFLOGINFO("DS2438: Salimos de readPage");
}

/** Escribe el vector privado "memPageData" en la pgina de memoria que
 * se indique.
 * Adems permite especificar el nmero total de bytes que se desea
 * transferir, por si no se quiere copiar toda la memPageData
 */
void DS2438::writeToPage(unsigned pageNum, unsigned numberOfBytes) {
  DEFLOGINFO("DS2438: Entramos en writeToPage");

  // Indicamos que vamos a escribir en la pgina-borrador #0
  master1W.reset1W( );
  skipRom();
  master1W.write1WByte(WR_SCRPAGE);
  master1W.write1WByte(pageNum);

  /* Una pequea medida de seguridad para no exceder el tamao del vector
  El "-1" es debido a que el comando write del integrado slo puede
  escribir 8 bytes, aunque el de lectura pueda con 9 */
  if (numberOfBytes > PAGELENGTH - 1) numberOfBytes=PAGELENGTH - 1;
  for (int i=0; i< numberOfBytes; i++)
    master1W.write1WByte(paginaInterna.memPageData[i]);

  // Finalmente indicamos que la scratchpad se transfiera a la eprom.
  master1W.reset1W( );
  skipRom();
  master1W.write1WByte(CPY_SCRPAGE);
  master1W.write1WByte(pageNum);

  DEFLOGINFO("DS2438: Salimos de writeToPage");
}

/**
 * \fn DS2438::convert(float *temp, float *volt, float *curr)
 * 
 * Convierte los valores de la temperatura, tensin y corriente
 * siguiendo las especificaciones del datasheet.
 * Hace uso de la informacin almacenada en la vector: memPageData[]
 * por lo que ha debido de ser previamente cargado.
 */
void DS2438::convert(float *temp, float *volt, float *curr) {

  DEFLOGINFO("DS2438: Entramos en convert");

  DEFLOGTRACE(
      ((NString)("DS2438: Valor de voltLSB->")).\
      nAppend("%#X", paginaInterna.voltLSB)
             );
  DEFLOGTRACE(
      ((NString)("DS2438: Valor de voltMSB->")).\
      nAppend("%#X", paginaInterna.voltMSB)
             );

  // ----- TENSION -----
  // La tensin se obtiene de manera directa uniendo los MSB y LSB,
  // pero teniendo en cuenta que slo se emplean 2 bits del MSB y
  // y que la unidad base son 10 mV.
  *volt = ((float)(paginaInterna.voltLSB + \
                      paginaInterna.voltMSB * 256)) / 100;

  DEFLOGTRACE(
      ((NString)("DS2438: Valor calculado de volt->")).\
      nAppend("%#X", *volt)
             );

  // ----- TEMPERATURA -----
  // Si el MSB bit-7 esta a 1, la temperatura tiene signo negativo.
  int signTemp= 1;
  const float PRECISION_TEMP=0.03125;
  if (paginaInterna.tempMSB & 0x80) signTemp= -1;
  // Hay que excluir el bit-7 MSB del calculo del valor de T.
  // Cuando el signo es negativo hay que hacer un complemento a 2
  // del valor.
  // En ambos casos hay que excluir los bits no utilizados del LSB y
  // el bit de signo.
  unsigned int T_conver= 0;
  T_conver= ((paginaInterna.tempMSB & 0x7F) * 256 + \
                  paginaInterna.tempLSB) >> 3;
  if (signTemp < 0 ) T_conver= (( ~T_conver ) - 1) & 0xFFF;

  *temp= signTemp * PRECISION_TEMP * T_conver;

  DEFLOGTRACE(
      ((NString)("DS2438: Valor de tempLSB->")).\
      nAppend("%#X", paginaInterna.tempLSB)
             );
  DEFLOGTRACE(
      ((NString)("DS2438: Valor de tempMSB->")).\
      nAppend("%#X", paginaInterna.tempMSB)
             );
  DEFLOGTRACE(
      ((NString)("DS2438: Valor calculado de temp->")).\
      nAppend("%#X", *temp)
             );

  // ----- CORRIENTE -----
  // La corriente se calcula usando:
  // I = valor_registro / (4096 * resistencia)
  // Hay que eliminar los 6 bits superiores que representan el signo
  // de la corriente.
  unsigned int registroIntensidad= 0;
  registroIntensidad= (paginaInterna.currMSB * 256 + \
                              paginaInterna.currLSB) & 0x3FF;
  *curr= float(registroIntensidad) / (4096 * sensorResistance);

  DEFLOGTRACE(
    ((NString)("DS2438: Valor de currLSB->")).\
    nAppend("%#X", paginaInterna.currLSB)
             );
  DEFLOGTRACE(
    ((NString)("DS2438: Valor de currMSB->")).\
    nAppend("%#X", paginaInterna.currMSB)
             );
  DEFLOGTRACE(
      ((NString)("DS2438: Valor calculado de curr->")).\
      nAppend("%#X", *curr)
             );

  DEFLOGINFO("DS2438: Salimos de convert");
}

/** Limpia el vector-memoria auxiliar
 */
void DS2438::clearMemPageData(void){
  for (int i=0; i<PAGELENGTH; i++) paginaInterna.memPageData[i]= 0;
}

}

