/**
 *   File: pca9564.cpp
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#include "pca9564.h"
#include "logger.h"

namespace CiematCMS {

//**********************************************************
//*********** CONSTRUCTORES Y DESTRUCTORES **************
//**********************************************************
PCA9564::PCA9564(I_VMEMaster &master) 
  : I_I2CMaster(), I_VMESlave (master) 
{
  DEFLOGINFO("Creamos un PCA9564");

  sioStarted= false;

  // Se establece por defecto la mxima frecuencia posible (320 kHz).
  frecuency=0;
  controlRegItem = statusRegItem = dataRegItem = ownAddRegItem = "";
}

PCA9564::~PCA9564() {
  DEFLOGINFO("Destruimos un PCA9564");
}

//**********************************************************
//*********** MTODOS PBLICOS **************
//**********************************************************
/**
 * Desarrolla todo el protocolo del integrado para la lectura
 * de varios bytes a travs del bus I2C
 */
void PCA9564::readI2CBytes(unsigned char slaveAdd, unsigned char* datos,
                           unsigned int bytesNumber)
    throw (CMSException)
{
  DEFLOGINFO("PCA9564: Entramos en readI2CBytes");

  // Tres variables cucas para transferir los valores de los registros.
  unsigned char status, data, control;

  /* Iniciamos el oscilador del bus I2C en caso que est parado.
  Desde luego, si por otra va paramos el oscilador, sin acudir a esta
  clase, no lo reactivar jams, hasta que no sea reiniciado.
  Tal vez convenga introducir en la clase algn mtodo de RESET que
  reinicie el integrado y los atributos de la clase para paliar esto */
  if (!sioStarted) {
    /* Creamos el valor que ha de tener el registro de control juntando
    el valor de la FRECUENCIA programada y activando el bit ENSIO */
    control= (frecuency | ENABLESIO);
    masterVME.writeVME(controlRegItem, control);
    usleep (OSC_START_TIME);
    sioStarted=true;
  }

  // Envio de orden START.
  control= (frecuency | ENABLESIO | STARTFLAG);
  masterVME.writeVME(controlRegItem, control);
  waitSI();
  masterVME.readVME(statusRegItem, &status);
  /* Se aceptan como vlidos los estados "START TRANSMITTED" y el
  "REPEATED START" */
  if ((status != STSTARTOK) && (status != STSTARTOKREP))
    throw (CMSException(
        "PCA9564::readI2CBytes", "No es posible un START", status));

  /* Envio de SLAVE-ADD + READ-bit. La direccion de esclavo se desplaza
  un bit a la izquierda para ocupar los ms significativos
  Primero lo almacenamos */
  data= (slaveAdd << 1) + READBIT;
  masterVME.writeVME(dataRegItem, data);
  // Resetemos el bit SI para transferir
  control= (frecuency | ENABLESIO);
  masterVME.writeVME(controlRegItem, control);
  waitSI();
  masterVME.readVME (statusRegItem, &status);
  if (status != STSLVADDRECREADOK) throw (CMSException(
        "PCA9564::readI2CBytes", 
        "Error, ningn esclavo responde a la direccion enviada", status));

  /* Lectura de los "bytesNumber" indicados */
  for (int i=0; i< bytesNumber; i++) {
    /*Si no es el ltimo byte para leer, hay que hacer ACK
    al recibir. Slo en el ltimo byte *NO* se realiza ACK*/
    if (i< bytesNumber - 1) control= (frecuency | ENABLESIO | ASSERTACK);
    else control= (frecuency | ENABLESIO);

    masterVME.writeVME(controlRegItem, control);
    waitSI();
    masterVME.readVME(dataRegItem, &data);
    /* Deberamos leer y comprobar el STATUS pero de momento pasamos
    y guardamos directamente el valor en el array que nos han pasado.
    Para mayor claridad *no* usamos aritmtica de punteros */
    datos[i]=data;
  }

  // Orden STOP al bus.
  control= (frecuency | ENABLESIO | STOPFLAG);
  masterVME.writeVME(controlRegItem, control);

  masterVME.readVME (statusRegItem, &status);
  if (status != STSTOPOK) throw (CMSException(
        "PCA9564::readI2CBytes", 
        "Error, fallo al enviar la seal de STOP", status));

  DEFLOGINFO("PCA9564: Salimos de readI2CBytes");
}

/**
 * Escribe una lista de bytes consecutivamente a travs del bus I2C
 */
void PCA9564::writeI2CBytes(unsigned char slaveAdd, unsigned char* datos,
                            unsigned int bytesNumber)
    throw (CMSException)
{
  DEFLOGINFO("PCA9564: Entramos en writeI2CBytes");

  // Tres variables cucas para transferir los valores de los registros.
  unsigned char status, data, control;

  /* Iniciamos el oscilador del bus I2C en caso que est parado.
  Desde luego, si por otra va paramos el oscilador, sin acudir a esta
  clase, no lo reactivar jams, hasta que no sea reiniciado.
  Tal vez convenga introducir en la clase algn mtodo de RESET que
  reinicie el integrado y los atributos de la clase para paliar esto */
  if (!sioStarted) {
    /* Creamos el valor que ha de tener el registro de control juntando
    el valor de la FRECUENCIA programada y activando el bit ENSIO */
    control= (frecuency | ENABLESIO);
    masterVME.writeVME(controlRegItem, control);
    usleep (OSC_START_TIME);
    sioStarted=true;
  }

  // Envio de orden START.
  control= (frecuency | ENABLESIO | STARTFLAG);
  masterVME.writeVME(controlRegItem, control);
  waitSI();
  masterVME.readVME(statusRegItem, &status);
  if ((status != STSTARTOK) && (status != STSTARTOKREP))
    throw (CMSException(
        "PCA9564::writeI2CBytes", "No es posible un START", status));

  /* Envio de SLAVE-ADD + WRITE-bit. La direccion de esclavo se desplaza
  un bit a la izquierda para ocupar los ms significativos
  Primero lo almacenamos */
  data= (slaveAdd << 1) + WRITEBIT;
  masterVME.writeVME(dataRegItem, data);
  // Resetemos el bit SI para transferir
  control= (frecuency | ENABLESIO);
  masterVME.writeVME(controlRegItem, control);
  waitSI();
  masterVME.readVME (statusRegItem, &status);
  if (status != STSLVADDRECWRITOK) throw (CMSException(
        "PCA9564::writeI2CBytes",
        "Error, ningn esclavo responde a la direccion enviada", status));

  /* Lectura de los "bytesNumber" indicados */
  for (int i=0; i< bytesNumber; i++) {
    // Escribimos el dato en el reg. de datos.
    masterVME.writeVME(dataRegItem, datos[i]);
    // Damos la orden de transferir, reseteando el SI-bit.
    control= (frecuency | ENABLESIO);
    masterVME.writeVME(controlRegItem, control);
    waitSI();
    // Comprobamos que el STATUS es 0x28, sino... a petaaaarr!
    masterVME.readVME (statusRegItem, &status);
    if (status != STWRITEOK) throw (CMSException(
          "PCA9564::writeI2CBytes",
          "Error, casque en el envo de un byte", status));
  }

  // Orden STOP al bus.
  control= (frecuency | ENABLESIO | STOPFLAG);
  masterVME.writeVME(controlRegItem, control);

  masterVME.readVME (statusRegItem, &status);
  if (status != STSTOPOK) throw (CMSException(
        "PCA9564::writeI2CBytes",
        "Error, fallo al enviar la seal de STOP", status));


  DEFLOGINFO("PCA9564: Salimos de writeI2CBytes");
}

/**
 * Este mtodo es anlogo al de lectura de datos, salvo que *NO*
 * implementa todo el proceso (solo hasta la parte de envo de
 * direccin del esclavo) y no peta con excepcion en caso que el
 * esclavo no responda a la direccin enviada.
 * Los comentarios de aquel mtodo se omiten en este.
 *
 * El parametro "demora" se emplea para esperar entre 2 ciclos
 * (2 direcciones de esclavo) consecutivos un breve tiempo y evitar
 * problemas de "timing" debido a los tiempos de respuesta del bus I2C.
 */
void PCA9564::testI2CSlaves(SLAVEADDS lista, unsigned long demora){

  DEFLOGINFO("PCA9564: Entramos en testI2CSlaves");

  unsigned char status, data, control;
  int indice_lista=0;

  if (!sioStarted) {
    control= (frecuency | ENABLESIO);
    masterVME.writeVME(controlRegItem, control);
    usleep (OSC_START_TIME);
    sioStarted=true;
  }

  // Recorremos todo el espectro de posibles esclavos.
  for (unsigned char slaveTEMP=0x0; slaveTEMP <= 0x7F; slaveTEMP++) {
    // Envio de orden START.
    control= (frecuency | ENABLESIO | STARTFLAG);
    masterVME.writeVME(controlRegItem, control);
    waitSI();
    masterVME.readVME(statusRegItem, &status);
    if ((status != STSTARTOK) && (status != STSTARTOKREP))
      throw (CMSException(
          "PCA9564::testI2CSlaves", "No es posible un START", status));

    // Simulamos que vamos a leer de un cierto esclavo.
    data= (slaveTEMP << 1) + READBIT;
    masterVME.writeVME(dataRegItem, data);
    // Resetemos el bit SI para transferir
    control= (frecuency | ENABLESIO);
    masterVME.writeVME(controlRegItem, control);
    waitSI();
    masterVME.readVME (statusRegItem, &status);
    /* Si se acepta la direccin, es que hay un esclavo que responde y la
    almacenamos en el array */
    if (status == STSLVADDRECREADOK) lista[indice_lista++]=slaveTEMP;

    // Orden STOP al bus.
    control= (frecuency | ENABLESIO | STOPFLAG);
    masterVME.writeVME(controlRegItem, control);

    masterVME.readVME (statusRegItem, &status);
    if (status != STSTOPOK) throw (CMSException(
          "PCA9564::testI2CSlaves",
          "Error, fallo al enviar la seal de STOP", status));
  }

  DEFLOGINFO("PCA9564: Salimos de testI2CSlaves");
}

//**********************************************************
//*********** MTODOS GET Y SET **************
//**********************************************************
void PCA9564::setControlRegItem (string value){
  controlRegItem= value;
}
string PCA9564::getControlRegItem (void){
  return controlRegItem;
}

void PCA9564::setStatusRegItem (string value){
  statusRegItem= value;
}
string PCA9564::getStatusRegItem (void){
  return statusRegItem;
}

void PCA9564::setDataRegItem(string value){
  dataRegItem= value;
}
string PCA9564::getDataRegItem(void){
  return dataRegItem;
}

void PCA9564::setOwnAddRegItem (string value){
  ownAddRegItem= value;
}
string PCA9564::getOwnAddRegItem (void){
  return ownAddRegItem;
}

//**********************************************************
//*********** MTODOS PRIVADOS **************
//**********************************************************
/**
 * Hace "pooling" peridico del registro de control para
 * comprobar si el flag SI (bit-3) est HIGH. En cuyo caso
 * se ha completado la transmisin/recepcin y finaliza el mtodo
 *
 * En caso que transcurra mucho tiempo desde que se entr en el
 * mtodo, se aborta con una excepcin por TIMEOUT.
 */
void PCA9564::waitSI(void) throw (CMSException){

  unsigned char control=0;

  struct timeval tinicial;
  tiempoAuxiliar.getActualTime(&tinicial);

  do {
    if (tiempoAuxiliar.utimeElapsed(&tinicial) > WAITSI_TIMEOUT)
      throw (CMSException("PCA9564::waitSI",
          "TIMEOUT esperando el flag SI.", 1000));

    usleep(ESPERA_WAITSI);
    masterVME.readVME(controlRegItem, &control);

  } while (!(control & SERIALINT));
}

}
