/**
 *   File: roscard.cpp
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#include "roscard.h"
#include "nstring.h"

namespace CiematCMS {

//**********************************************************
//*********** CONSTRUCTORES Y DESTRUCTORES **************
//**********************************************************
/** Contruye todos los componentes de la tarjeta.
 * El orden como se disponen es importante para poder enlazarlos
 * segun requieren sus respectivos constructores.
 */
ROSCard::ROSCard(VMEDevice &deviceIn)
  : I_VMEMaster(), device(deviceIn),
    rosPCA9564( (*this) ),
    rosDS2482(rosPCA9564),
    rosDS2438(rosDS2482)
{
  DEFLOGINFO("Creamos un objeto ROSCard");

  // Configuramos los "ITEMs"
  rosPCA9564.setControlRegItem(I2CCONTROLITEM);
  rosPCA9564.setDataRegItem(I2CDATAITEM);
  rosPCA9564.setStatusRegItem(I2CSTATUSITEM);

  /* Si hubiera varios esclavos unidos al bus, esta llamada convendra
  hacerla antes de peticin de informacin al esclavo correpondiente.
  Puesto que todos los sensores de la ROS comparten la misma direccin de
  esclavo, por estar en distintos canales del DS2482, podemos efectuar
  la invocacin en el constructor y ahorramos llamadas */
  rosDS2482.setI2CSlaveAddress(ds2482_I2CSlaveADD);

}

ROSCard::~ROSCard()
{
  DEFLOGINFO("Destruimos un objeto ROSCard");
}

//**********************************************************
//*********** MTODOS PBLICOS **************
//**********************************************************
void ROSCard::writeVME (string vmeAdd, unsigned char byte)
    throw (CMSException)
{
  device.write(vmeAdd, (unsigned long)byte);

  DEFLOGTRACE(
    ((NString)("ROSCard.writeVME: byte escrito->")).\
    nAppend("%#X", (unsigned long)byte)
             );
}

void ROSCard::readVME (string vmeAdd, unsigned char* byte)
    throw (CMSException)
{
  // Variable til para la conversin de tipo.
  unsigned long valor=0;
  //uint32_t valor=0;

  device.read(vmeAdd, &valor);
  // Transcribimos el valor ledo al parmetro entrante.
  *byte=(unsigned char)valor;

  DEFLOGTRACE(
    ((NString)("ROSCard.readVME: byte leido->")).\
    nAppend("%#X", *byte));
}

/** Consulta al sensor correspondiente y devuelve el tiempo que
 * lleva encendido.
 *
 * (Ver comentarios del mtodo "getTempVoltCurr")
 */
unsigned long ROSCard::getSensorElapsedTime(SENSORS_ID sensor){

  DEFLOGINFO("ROSCard: Entramos en getSensorElapsedTime");

  unsigned long time= 0;

  habilitarSensores();
  switch (sensor) {
    case SENSID5:
      rosDS2482.setChannel(DS2482::CH1);
      break;
    case SENSID3_3:
      rosDS2482.setChannel(DS2482::CH2);
      break;
    case SENSID1_8:
      rosDS2482.setChannel(DS2482::CH3);
      break;

    default:
      rosDS2482.setChannel(DS2482::CH1);
  }

  time=rosDS2438.getElapsedTime();

  DEFLOGINFO("ROSCard: Salimos de getSensorElapsedTime");

  return time;
}

/** Obtiene los parmetros del sensor que se indique.
 */
void ROSCard::getTempVoltCurr(SENSORS_ID sensor,
                              float *temp, float *volt, float *current)
{
  DEFLOGINFO("ROSCard: Entramos en getTempVoltCurr");

  // A la ROS le indicamos que habilite el acceso a los sensores
  // a travs del PCA9516 y que inhabilite el acceso al GOL
  habilitarSensores();

  /* Seleccionamos la entrada VAD o VDD (segn la tensin linea de
  tensin que se vaya a medir) del DS2438 y fijamos el canal del DS2482
  que queremos leer.

  Adems indicamos por separado el valor de la resistencia conectada al
  sensor.
  OJO! Esto se podra hacer en el constructor de RosCARD, pero obligara
  a que todos los posibles sensores de la tarjeta tuvieran conectada
  la misma resistencia. De momento esto es as, pero puede cambiar con
  la versin de la misma */
  rosDS2438.setSensorResistance(resistenciaSensor);

  switch (sensor) {
    case SENSID5:
      rosDS2438.selectVADInput();
      rosDS2482.setChannel(DS2482::CH1);
      break;
    case SENSID3_3:
      rosDS2438.selectVDDInput();
      rosDS2482.setChannel(DS2482::CH2);
      break;
    case SENSID1_8:
      rosDS2438.selectVADInput();
      rosDS2482.setChannel(DS2482::CH3);
      break;

    default:
      rosDS2438.selectVADInput();
      rosDS2482.setChannel(DS2482::CH1);
  }
  rosDS2438.getTempVoltCurr(temp, volt, current);

  DEFLOGINFO("ROSCard: Salimos de getTempVoltCurr");
}

/** Habilita el acceso a los sensores.
 *
 * Permite acceder a los sensores DS2438 y DS2450 escribiendo en el
 * registro GOL/QPLL un 0 en el bit-0 y un 1 en el bit-1.
 * No modifica el resto de los bits.
 *
 * GOLQPLL ha de estar definido en el AddressTable
 */
void ROSCard::habilitarSensores(void) throw (CMSException){

  DEFLOGINFO("ROSCard: Entramos en habilitarSensores");

  unsigned char registro='\0';

  this->readVME(GOLQPLLCTRL, &registro);
  // Ponemos a cero el bit-0 y a 1 el bit-1.
  registro= ((registro & 0xFE) | 0x02);
  this->writeVME(GOLQPLLCTRL, registro);

  DEFLOGINFO("ROSCard: Salimos de habilitarSensores");
}

}


