/**
 *   File: configfileparser.cpp
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#include "configfileparser.h"

namespace CiematCMS {

//**********************************************************
//*********** CONSTRUCTORES Y DESTRUCTORES **************
//**********************************************************
ConfigFileParser::ConfigFileParser(string fileName){
  ConfigFileParser();
  inputFilename = fileName;
}

ConfigFileParser::ConfigFileParser() : I_ConfigTree()
{

  DEFLOGINFO("Creamos un objeto ConfigFileParser");

  totalLines = 0;
  totalSections = 0;
  caseSensStatus = CASE_SENS; // Por defecto distinguimos mayus/minus
  caseConvert = LEAVE_CASE; //.. y no tocamos las variables.
}

ConfigFileParser::~ConfigFileParser() {
  DEFLOGINFO("Destruimos un objeto ConfigFileParser");
}

//**********************************************************
//*********** MTODOS PBLICOS **************
//**********************************************************
void ConfigFileParser::saveFile(string path) throw (CMSException){
  throw (CMSException ("ConfigFileParser::saveFile",
         "Mtodo sin implementar", 100));
}

/** Procede al anlisis del fichero cuyo nombre se ha especificado
 * previamente.
 */
void ConfigFileParser::parseFile(void)
    throw (CMSException)
{
    DEFLOGINFO("ConfigFileParser: Entramos en parseFile");

    /* Intentamos abrir el fichero para el anlisis */
  try {
    inputFile.open(inputFilename.c_str());
  }
  catch (std::exception e){
    throw (CMSException("ConfigFileParser::parseFile",NString(
        "No es posible abrir el fichero: ").sAppend(inputFilename),
        -100));
  }

  /* Iterator conveniente para el proceso del bucle */
  map<string, ConfigFileSection>::iterator itCurrentSection;

  /* Recorremos el fichero lnea a lnea */
  string currentLine, literal = "", value = "";
  unsigned int lineType;
  while (! inputFile.eof() ){
    totalLines++;

    /* Leemos una lnea */
    getline(inputFile, currentLine);
    /* Analizamos la linea */
    lineType= parseLine(currentLine, literal, value);

    /* Decidimos que hacer segn se trate de seccin o variable */
    switch (lineType) {
      case SECTION:
        if (sectionMap.find(literal) != sectionMap.end())
          sectionMap.erase(literal);

        sectionMap[literal] = ConfigFileSection(literal);
        totalSections ++;
        itCurrentSection = sectionMap.find(literal);
        break;

      case VARIABLE:
        if (totalSections == 0) addRootVar(literal, value);
        else (itCurrentSection->second).addVar(literal, value);
        break;

      case ERROR:
        inputFile.close();
        throw (CMSException ("ConfigFileParser::parseFile",
          NString("Error en lnea: ").append("%d", totalLines),
          totalLines));
    }
  }

  /* Finalmente cerramos el fichero */
  inputFile.close();

  DEFLOGINFO("ConfigFileParser: Salimos en parseFile");
}

/** Recorta espacios por la derecha
 */
string ConfigFileParser::rightTrim (string value){

  while(value.at(value.length() - 1) == ' ')
    value.replace(value.length() - 1, 1, "");

  return value;
}

/** Recorta espacios por la izquierda
 */
string ConfigFileParser::leftTrim (string value){

  while(value.at(0) == ' ') value.replace(0, 1, "");

  return value;
}

/**
 * Implementa el interface "I_ConfigTree"
 */
string ConfigFileParser::getKeyValue(string key){

  return this->getValue(key);
}

//**********************************************************
//*********** MTODOS GET Y SET **************
//**********************************************************
/** Devuelve el valor de una variable dentro de una seccin.
 * El resultado se convierte a MAYUS o MINUS (o se deja como est),
 * segn la configuracin.
 */
string ConfigFileParser::getValue(string section, string variable)
    throw (CMSException)
{
  map<string, ConfigFileSection>::iterator iter =
      sectionMap.find( analizeCaseSens(section) );

  if (iter != sectionMap.end())
    return  convertCase(
        (iter->second).getVarValue( analizeCaseSens(variable) )
                       );
  else throw (CMSException ("ConfigFileParser::getValue",
              NString("No se localiza la variable: ")\
                  .append(variable).append(" en la seccion: ")\
                  .append(section), 100));
}

/** Devuelve el valor de una variable SIN seccin.
 * El resultado se convierte a MAYUS o MINUS (o se deja como est),
 * segn la configuracin.
 */
string ConfigFileParser::getValue(string variable)
    throw (CMSException)
{
  return convertCase( rootVarMap[ analizeCaseSens(variable) ] );
}

void ConfigFileParser::setValue(string section, string variable,
                                string value)
    throw (CMSException)
{
  throw (CMSException ("ConfigFileParser::setValue",
         "Mtodo sin implementar", 100));
}

void ConfigFileParser::setInputFile(string filename)
    throw (CMSException)
{
  inputFilename = filename;
}

void ConfigFileParser::setCaseSensitive(void){
  caseSensStatus =CASE_SENS;
}
void ConfigFileParser::unsetCaseSensitive(void){
  caseSensStatus =NO_CASE_SENS;
}

void ConfigFileParser::setVar2Upper(void){
  caseConvert = CONVERT_UPPER;
}
void ConfigFileParser::setVar2Lower(void){
  caseConvert = CONVERT_LOWER;
}
void ConfigFileParser::setOwnCase(void){
  caseConvert = LEAVE_CASE;
}

/** Devuelve el nmero de secciones localizadas en el fichero
 */
unsigned int ConfigFileParser::getTotalSections(void){
  return sectionMap.size();
}

/** Devuelve el nmero de variables localizadas *SIN* seccin 
 */
unsigned int ConfigFileParser::getTotalRootVars(void){
  return rootVarMap.size();
}

/** Devuelve el total de variables en una seccin dada
 */
unsigned int ConfigFileParser::getTotalVarInSec(string section){
  return (sectionMap[section]).getTotalVar();
}


//**********************************************************
//*********** MTODOS PRIVADOS **************
//**********************************************************
/** Analiza una lnea indicando si es errnea, si contiene
 * el nombre de una seccin o si se trata de una variable.
 * 
 * En el caso de SECCIN devuelve un "literal" con su nombre
 * habiendo eliminado los "blancos" por los lados.
 *
 * En el caso de una VARIABLE devuelve el "literal" con el nombre
 * y el "value" con el valor.
 *
 * El "literal" se tranformar en ambos casos a MAYUS en caso que se haya
 * determinado *NO* ser sensible a MAYUS/MINUS, de manera que se almacene
 * todo en MAYSCULAS para no hacer distinciones.
 */
int ConfigFileParser::parseLine(string line, string &literal,
                                     string &value)
{
  DEFLOGINFO("ConfigFileParser: Entramos en parseLine");

  string::size_type posIni = 0, posFin = 0;
  string literalTemp, valueTemp;

  /* Vaciamos inicialmente el contenido de las referencias de entrada */
  literal = "";
  value = "";

  if ( line.empty() ) return COMMENT;

  line = leftTrim(line);
  /* Nos situamos en el primer carcter no-blanco */
  posIni = line.find_first_not_of(" ");
  /* Verificamos que no sean todo blancos. En caso contrario salimos */
  if (posIni == string::npos) return COMMENT;

  /* Buscamos una posible marca de comentario */
  posFin = line.find_first_of("#");
  /* ... si est al inicio, toda la lnea es comentario */
  if (posFin == 0) return COMMENT;
  else
    /* ... si no aparece la marca "#", "posFin" debe coincidir con el
    final de la lnea; en caso contrario lo disminuimos en 1 para evitar
    la propia marca "#" */
    if (posFin == string::npos) posFin = line.length() - 1;
    else posFin--;

  /* Aqu intentamos identificar si es una seccin */
  if (line[posIni] == '[') { // Hay marca de inicio seccin

    /* Si no encontramos el "]" devolvemos error */
    posFin = line.find_last_of("]");
    if (posFin == string::npos) return ERROR;
    /* Extraemos la cadena de inters */
    literalTemp = line.substr(posIni + 1, posFin - posIni - 1);
    /* Validamos sintcticamente la cadena */
    if (validateLiteral( literalTemp ) != ERROR) {
      /* ... y la devolvemos bien recortadita de blancos, promocionndola
      a MAYUS en caso que no haya "case sensitive" */
      literal = analizeCaseSens( rightTrim( leftTrim(literalTemp) ) );
      return SECTION;
    }
    else return ERROR;
  }
  /* Llegado aqu, suponemos que es una variable y sino error */
  else {
    line = line.substr(posIni, posFin - posIni + 1);
    /* Primero buscamos desde el inicio hasta el smbolo igual (=)*/
    string::size_type posEqual = line.find_first_of("=");
    /* ... si es una variable y no encontramos el "=", error */
    if (posEqual == string::npos) return ERROR;

    /* Usando el "=" como punto central, separamos el literal y su valor */
    literalTemp = line.substr(0, posEqual);
    valueTemp   = line.substr(posEqual + 1, line.length() - posEqual);
    if (validateLiteral(literalTemp) == ERROR ||
        validateLiteral(valueTemp)   == ERROR ) return ERROR;

    /* Cambiamos a MAYUS el literal si no se hace "case sensitive" */
    literal = analizeCaseSens( rightTrim( leftTrim(literalTemp) ) );
    value   = rightTrim( leftTrim(valueTemp) );

    return VARIABLE;
  }

}

/** Verifica que no contiene ninguno de los caracteres prohibidos en un
 * literal. La lista de los mismos est definida en __LITERALES_INVALIDOS__
 */
int ConfigFileParser::validateLiteral(string literal){
  /* Comprobamos que no sean todo blancos */
  if (literal.find_last_of(" ") == literal.length()) return ERROR;
  /* ... y que no sean caracteres prohibidos */
  if (literal.find_first_of(__LITERALES_INVALIDOS__) != string::npos)
    return ERROR;
  else return OK;
}

/** Agrega una variable sin seccin a la lista ROOT
 */
void ConfigFileParser::addRootVar(string varName, string varValue)
    throw (CMSException)
{
  rootVarMap[varName] = varValue;
}

/** Transforma un string a MAYSCULAS
 */
string ConfigFileParser::toUpper(string cadena){
  string tmp = cadena;

  /* Tenemos que indicar el "cast" para evitar que la funcin "transform"
  confunda la llamada a "tolower" de "cctype" y la de "locale" */
  transform (tmp.begin(), tmp.end(), tmp.begin(),
             static_cast < int(*)(int) > (toupper));
  return tmp;
}

/** Transforma un string a MINSCULAS
 */
string ConfigFileParser::toLower(string cadena){
  string tmp = cadena;

  /* Tenemos que indicar el "cast" para evitar que la funcin "transform"
  confunda la llamada a "tolower" de "cctype" y la de "locale" */
  transform (tmp.begin(), tmp.end(), tmp.begin(),
             static_cast < int(*)(int) > (tolower));
  return tmp;
}

/** Segn la configuracin de conversin, convierte la cadena a UPPER,
 * LOWER o la deja como est.
 */
string ConfigFileParser::convertCase(string cadena){
  if (caseConvert == CONVERT_UPPER) return toUpper(cadena);
  else if (caseConvert == CONVERT_LOWER) return toLower(cadena);
  else return cadena;
}

/** Si se hay que distinguir MAYUS/MINUS pasamos todo a MAYUS.
 */
string ConfigFileParser::analizeCaseSens(string cadena){
  if (caseSensStatus == CASE_SENS) return cadena;
  else return toUpper(cadena);
}

}
