/**
 *   File: ds2438.cpp
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#include "ds2438.h"

namespace CiematCMS {

//**********************************************************
//*********** CONSTRUCTORES Y DESTRUCTORES **************
//**********************************************************
DS2438::DS2438(I_OneWireMaster &master)
  : Dallas_1WireSlave(master) 
{
  sensorResistance= 0;
  for (int i=0; i<PAGELENGTH; i++) pagina0.memPageData[i]= 0;
}

DS2438::~DS2438()
{
}

//**********************************************************
//*********** MTODOS PBLICOS **************
//**********************************************************
/*!
 * \fn DS2438::getTempVoltCurr(float *temp, float *volt, float *curr)
 */
void DS2438::getTempVoltCurr(float *temp, float *volt, float *curr) {
  // Inciamos el ciclo de dilogo con el integrado.
  master1W.reset1W( );
  skipRom( );

  // Este sera un buen sitio para configurar el integrado.

  // Enviamos orden de conversion de temperatura y tension.
  master1W.reset1W( );
  skipRom( );
  master1W.write1WByte(CONVERT_T);
  master1W.reset1W( );
  skipRom( );
  master1W.write1WByte(CONVERT_V);

  // Esperamos a que se produzca la conversin.
  usleep (WTIMECONVERT);
  // Leemos la pagina de memoria #0. Los datos se almacenan 
  readPage (0);
  convert (temp, volt, curr);
}

//**********************************************************
//*********** MTODOS GET Y SET **************
//**********************************************************
float DS2438::getSensorResistance() const {
  return sensorResistance;
}

void DS2438::setSensorResistance(float dato) {
  sensorResistance = dato;
}

//**********************************************************
//*********** MTODOS PRIVADOS **************
//**********************************************************
/*!
 * \fn DS2438::readPage (unsigned pageNum)
 * 
 * Lee una pagina de memoria del integrado y la vuelca en el
 * vector privado "memPageData"
 */
void DS2438::readPage (unsigned pageNum) {
  // Recall page #0 to scratchpad page #0
  // Copiamos la pgina de memoria en una pgina de trabajo.
  master1W.reset1W();
  skipRom();
  master1W.write1WByte(RCLL_MEMPAGE);
  master1W.write1WByte(0);

  // Read scratchpad page #0
  // Indicamos que vamos a chupar los 9 bytes de la pgina 0
  // de memoria. Despus es obligatorio leerlos. Consultar datasheet.
  master1W.reset1W();
  skipRom();
  master1W.write1WByte(RD_SCRPAGE);
  master1W.write1WByte(0);

  // Leemos los datos de la memPage
  for (int i=0; i< PAGELENGTH; i++)
    pagina0.memPageData[i] = master1W.read1WByte();
}

/**
 * \fn DS2438::convert(float *temp, float *volt, float *curr)
 * 
 * Convierte los valores de la temperatura, tensin y corriente
 * siguiendo las especificaciones del datasheet.
 * Hace uso de la informacin almacenada en la vector: memPageData[]
 * por lo que ha debido de ser previamente cargado.
 */
void DS2438::convert(float *temp, float *volt, float *curr) {

  // ----- TENSION -----
  // La tensin se obtiene de manera directa uniendo los MSB y LSB,
  // pero teniendo en cuenta que slo se emplean 2 bits del MSB y
  // y que la unidad base son 10 mV.
  *volt = (pagina0.voltLSB + pagina0.voltMSB * 256) / 100;

  // ----- TEMPERATURA -----
  // Si el MSB bit-7 esta a 1, la temperatura tiene signo negativo.
  int signTemp= 1;
  const float PRECISION_TEMP=0.03125;
  if (pagina0.tempMSB & 0x80) signTemp= -1;
  // Hay que excluir el bit-7 MSB del calculo del valor de T.
  // Cuando el signo es negativo hay que hacer un complemento a 2
  // del valor.
  // En ambos casos hay que excluir los bits no utilizados del LSB y
  // el bit de signo.
  unsigned int T_conver= 0;
  T_conver= ((pagina0.tempMSB & 0x7F) * 256 + pagina0.tempLSB) >> 3;
  if (signTemp < 0 ) T_conver= (( ~T_conver ) - 1) & 0xFFF;

  *temp= signTemp * PRECISION_TEMP * T_conver;

  // ----- CORRIENTE -----
  // La corriente se calcula usando:
  // I = valor_registro / (4096 * resistencia)
  // Hay que eliminar los 6 bits superiores que representan el signo
  // de la corriente.
  unsigned int registroIntensidad= 0;
  registroIntensidad= (pagina0.currMSB * 256 + pagina0.currLSB) & 0x3FF;
  *curr= float(registroIntensidad) / (4096 * sensorResistance);
}

}

