/**
 *   File: ds2482.cpp
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#include "ds2482.h"
#include "timegeneral.h"

namespace CiematCMS {

//**********************************************************
//*********** CONSTRUCTORES Y DESTRUCTORES **************
//**********************************************************
DS2482::DS2482(I_I2CMaster &master)
  : I_OneWireMaster(), I_I2CSlave(master) {
  currentChannel= CH0;
}

DS2482::~DS2482() {
}

//**********************************************************
//*********** MTODOS PBLICOS **************
//**********************************************************
/**
 * Lee un byte a travs del canal seleccionado por defecto.
 * Debe controlar el protocolo y esperar a que la actividad
 * en el bus haya finalizado (bit 1WB == 0) antes de devolver
 * el control.
 */
unsigned char DS2482::read1WByte (void) throw (CMSException){
  // Array con el comando READ_BYTE.
  unsigned char read1wCmd[1]= {char(READ_BYTE)};

  // Ejecutamos el comando "leer"
  masterI2C.writeI2CBytes(read1wCmd, 1);
  waitUntil_1W_Stop();
  // Fijamos el registro de datos para leerlo. Usamos un array
  // de 2 bytes: COMANDO + Registro-elegido
  unsigned char selectDataRegCmd[2]={char(SET_READPOINTER) , '\0'};
  selectDataRegCmd[1]=char(DATAREG);
  masterI2C.writeI2CBytes(selectDataRegCmd, 2);
  // Leemos el dato
  unsigned char dataByte[1]={'\0'};
  masterI2C.readI2CBytes(dataByte, 1);

  return dataByte[0];
};

void DS2482::write1WByte (unsigned char byte) throw (CMSException){
  // Array con el comando WRITE_BYTE y con espacio para el byte
  // que se va a escribir.
  unsigned char write1wCmd[2]= {char(WRITE_BYTE), '\0'};
  write1wCmd[1]=byte;

  masterI2C.writeI2CBytes(write1wCmd, 2);
  waitUntil_1W_Stop();
  
};

void DS2482::reset1W (void) throw (CMSException){
  // Array con el comando RESET_1W.
  unsigned char reset1wCmd[1]= {char(RESET_1W)};

  masterI2C.writeI2CBytes(reset1wCmd, 1);
  waitUntil_1W_Stop();
};

//**********************************************************
//*********** MTODOS GET Y SET **************
//**********************************************************
unsigned DS2482::getChannel() const {
    return currentChannel;
    // Convendra meter cdigo para verificar qu canal es
    // el que el integrado "cree" tener activo.
}

void DS2482::setChannel(enum CHANNEL_CODES dato)
    throw (CMSException) {
   // Array para almacenar la orden de cambio de canal.
  unsigned char selChannelCmd[2]= {char(SELECT_CHANNEL), '\0'};
  // Para verificar el canal seleccionado
  unsigned char chVerif[1];
 
  selChannelCmd[1]=dato;

  // Fijamos el canal y verificamos que se corresponde con el
  // elegido. En caso contrario, mandamos una excepcin.
  masterI2C.writeI2CBytes(selChannelCmd, 2);
  masterI2C.readI2CBytes(chVerif, 1);

  // Preparamos el mensaje para la posible excepcin.
  string mensajeExcp= "No se puede fijar el canal elegido (";
  char canalTXTTemp[30];
  sprintf (canalTXTTemp, "%#X", dato);
  mensajeExcp+= canalTXTTemp;
  mensajeExcp+= "). Respuesta: ";
  sprintf (canalTXTTemp, "%#X", chVerif[0]);
  mensajeExcp+= canalTXTTemp;
  
  if ((dato == CH0 && chVerif[0] != CH0_RESP) ||
      (dato == CH1 && chVerif[0] != CH1_RESP) ||
      (dato == CH2 && chVerif[0] != CH2_RESP) ||
      (dato == CH3 && chVerif[0] != CH3_RESP) ||
      (dato == CH4 && chVerif[0] != CH4_RESP) ||
      (dato == CH5 && chVerif[0] != CH5_RESP) ||
      (dato == CH6 && chVerif[0] != CH6_RESP) ||
      (dato == CH7 && chVerif[0] != CH7_RESP)) throw (
        CMSException("DS2482::setCurrentChannel", mensajeExcp, 2)
        );
  else currentChannel = dato;
}

//**********************************************************
//*********** MTODOS PRIVADOS **************
//**********************************************************
/*!
 * \fn CiematCMS::DS2482::waitUntil_1W_Stop (void)
 * Aguarda hasta que el bit 1WB se fija a LOW. Mientras est
 * HIGH indica que el bus 1-W est ocupado transmitiendo informacin.
 * Se realizan ciclos de polling en cada uno de los cuales
 * se comprueba el valor del bit.
 * Entre ciclos se espera un tiempo prefijado.
 */
void DS2482::waitUntil_1W_Stop (void) throw (CMSException){
  // Vector para almacenar el registro de estado.
  unsigned char status[]={0};
  // Vector con el comando para apuntar al registro de estado
  unsigned char pointToStatusCmd[2]= {
    char(SET_READPOINTER),char(STATUSREG)
  };

  // Para obtener la hora actual y operar con microsegundos con
  // comodidad.
  struct timeval tinicial;
  tiempoAuxiliar.getActualTime(&tinicial);

  do {
    // Fijamos el puntero de lectura al registro de STATUS, y lo leemos
    masterI2C.writeI2CBytes(pointToStatusCmd, 2);
    masterI2C.readI2CBytes(status ,1);
    usleep (WAITTIME1WBUSY);

    // Abortamos el bucle en caso de demasiada tardanza, con 
    // una bonita excepcion.
    if (tiempoAuxiliar.utimeElapsed(&tinicial) > TIMEOUT)
      throw (
          CMSException("DS2482::waitUntil_1W_Stop", "Timeout", 1000)
          );

  } while ((status[0] & WIREBUSY)); // Repetimos hasta que libere el BUSY.
}

}
