/**
 *   File: ds2438.h
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#ifndef CIEMATCMSDS2438_H
#define CIEMATCMSDS2438_H

#include "dallas_1wireslave.h"
#include "logger.h"

namespace CiematCMS {

/**
 * Implementacin de las primitivas tiles del sensor DS2438
 *
 * @author J.M. <josemanuel.cela@ciemat.es>
 */
class DS2438 : public Dallas_1WireSlave {

  public:
    DS2438(I_OneWireMaster &master);
    ~DS2438();
    void getTempVoltCurr (float *temp, float *volt, float *curr);
    unsigned long getElapsedTime (void);

    void setSensorResistance(float dato);
    float getSensorResistance() const;

    void enableCurrentAD(void);
    void disableCurrentAD(void);

    void selectVADInput(void);
    void selectVDDInput(void);

  private:
    /*  Duracin mxima estimada para cualquier conversin.
        Se usar como tiempo de espera en ciertos mtodos, para
        aguardar a que acabe la conversin de la temperatura o de la
        tensin.
        Est medido en microsegundos.
    */
    static unsigned int const WTIMECONVERT=50000;
    /* Longitud de una pagina de memoria en bytes. Dispone de 1 bytes
    adicional para poder almacenar el CRC suministrado durante la
    lectura */
    static unsigned int const PAGELENGTH=9;

    /* Cdigos de los distintos comandos que admite el integrado */
    enum CMD_CODES {
      CPY_SCRPAGE=0x48, CONVERT_T=0x44,
      CONVERT_V=0xB4,
      RCLL_MEMPAGE=0xB8, RD_SCRPAGE=0xBE,
      WR_SCRPAGE=0x4E
    };

    /* Valores hex tiles para fijar a 1 el valor de los distintos
    bits del CONTROL/STATUS mediante "bit-or".
    Slo se definen los que se estn usando en el cdigo */
    enum CONTROL_BITS {
      CTRLBIT_IAD=0x01,
      CTRLBIT_AD=0x08
    };

    /* Valor de la resistencia conectada al sensor de corriente.
    Consultar datasheet del integrado */
    float sensorResistance;
    /* Almacenan el estado de los bits correspondientes del registro
    de control-estado */
    bool bit_IAD;
    bool bit_AD;

    /* Flag para controlar si en algn momento se ha cambiado la
    configuracin y es necesario reconfigurar el integrado.
    Con l deberamos poder ahorrar algunos accesos.

      Se hace uso de este mecanismo, en vez de acceder al integrado
    cada vez que se modifica un bit de configuracin, puesto que puede ser
    que el master al que est conectado sea de tipo "multicanal" y se
    requiera haber seleccionado previemente el canal en aquel, antes de
    poder acceder al DS2438.

      De esta manera todos los mtodos propios de este integrado llamarn
    al mtodo de configuracin, el cual se encargar de configurar slo si
    es estrictamente necesario.
      As, la responsabilidad del programador se limita a la seleccin
    previa (si ha lugar) del canal en el master y despus a llamar a los
    mtodos pblicos de esta clase */
    bool configChanged;

    /* Vector para el almacenamiento de una pagina de memoria
    Mediante una union se le aade un molde para extraer comodamente
    los bytes de datos que lo forman.
    Todas las "struct" tienen un byte menos que la longitud de la pgina
    puesto que no se incluye el CRC en ellas.

    Se aaden tantas "struct" a la "union" como formatos de pgina
    sean necesarios. Cada pgina del integrado tiene un significado
    diferente. Ver el datasheet. */
    union {
      unsigned char memPageData [PAGELENGTH];

      // Datos almacenados en la pgina 0.
      struct {
        unsigned char statConf, tempLSB, tempMSB,
        voltLSB, voltMSB,
        currLSB, currMSB,
        threshold;
      };

      // Datos almacenados en la pgina 1.
      struct {
        unsigned char elapsedTime0, elapsedTime1,
        elapsedTime2, elapsedTime3,
        ICA,
        offsetLSB, offsetMSB,
        RESERVED_1;
      };
    } paginaInterna;

    // Configura los bits del registro de control.
    void configDevice(void);
    void convert (float *temp, float *volt, float *curr);
    void readPage (unsigned pageNum);
    void writeToPage (unsigned pageNum, unsigned numberOfBytes);
    // Limpia el vector auxiliar
    void clearMemPageData(void);
};

}

#endif
