/**
 *   File: ds2482.h
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#ifndef CIEMATCMSDS2482_H
#define CIEMATCMSDS2482_H

#include "i_onewiremaster.h"
#include "i_i2cslave.h"
#include "timegeneral.h"
#include "logger.h"
#include <unistd.h>

namespace CiematCMS {

// Codigos de seleccin de canal.

/**
 * Clase con los mtodos bsicos que proporciona el integrado DS2482-800.
 * Acta como interfaz entre el bus I2C y el bus 1-Wire.
 *
 *  IMPORTANTE !!! Esta clase tiene pendiente detectar que *NO* hay
 * esclavos respondiendo a sus peticiones.
 * Esto puede ocurrir en caso de fallo hardware o en el caso que varios
 * esclavos estn conectados al bus y se acceda a ellos empleando sus
 * direcciones ROM, con alguna de ellas equivocada.
 *
 * @author J.M. <josemanuel.cela@ciemat.es>
*/
class DS2482 : public I_OneWireMaster, public I_I2CSlave {
  public:
    // Cdigos para la seleccin de canal.
    enum CHANNEL_CODES {
        CH0=0xF0, CH1=0xE1,
        CH2=0xD2, CH3=0xC3,
        CH4=0xB4, CH5=0xA5,
        CH6=0x96, CH7=0x87
    };
    // Cdigos para la verificacin del canal elegido.
    enum CHANNEL_READBACKS {
        CH0_RESP=0xB8, CH1_RESP=0xB1,
        CH2_RESP=0xAA, CH3_RESP=0xA3,
        CH4_RESP=0x9C, CH5_RESP=0x95,
        CH6_RESP=0x8E, CH7_RESP=0x87
    };

    DS2482(I_I2CMaster &master);
    ~DS2482();

    void setChannel (enum CHANNEL_CODES dato) throw (CMSException);
    unsigned getChannel() const;

    bool singleBit1WRead (void) throw (CMSException);
    void singleBit1WWrite (bool bitValue) throw (CMSException);
    unsigned char read1WByte (void) throw (CMSException);
    void write1WByte (unsigned char byte) throw (CMSException);
    void reset1W (void) throw (CMSException);

  private:
    // Cdigos hexadecimales de los diferentes comandos que
    // soporta el integrado.
    enum CMD_CODES {
      READ_BYTE=0x96, RESET_DEVICE=0xF0,
      RESET_1W=0xB4,
      SET_READPOINTER=0xE1, SELECT_CHANNEL=0xC3,
      WRITE_BYTE=0xA5,
      SINGLEBIT_OPER=0x87
    };
    // Mascaras para obtener los bits del registro de estado
    // mediante un "AND".
    enum STATUSREG_MASKS {
      BRANCHDIRTAKE=0x80,
      TRIPLESECONDBIT=0x40,
      SINGELBITRESULT=0x20,
      DEVICERESET=0x10,
      LOGICLEVEL=0x08,
      SHORTDETECTED=0x04,
      PRESENCEPULSE=0x02,
      WIREBUSY=0x01
    };
    // Codigos de los registros internos.
    enum REGISTER_CODE_POINTERS {
      STATUSREG=0xF0,
      DATAREG=0xE1,
      CHANNELSELREG=0xD2,
      CONFIGREG=0xC3
    };
    // Tiempo del ciclo de espera para ver si el bus 1-W est
    // libre despus de ejecutar el ltimo comando enviado.
    // Se mide en microsegundos.
    static unsigned int const WAITTIME1WBUSY=500;
    // Valor del timeout para evitar bucles infinitos y lanzar
    // una excepcion en caso que se exceda. En microsegundos.
    static unsigned long const TIMEOUT=3000000;
    // Creamos esta clase para usar sus mtodos y evitar su creacin
    // y destruccin dentro de los bucles que la emplean.
    TimeGeneral tiempoAuxiliar;

    /**
     * Almacena el canal 1-Wire en curso.
     */
    enum CHANNEL_CODES currentChannel;

    void waitUntil_1W_Stop(void) throw (CMSException);
    bool singleBitOperation(bool bitValue) throw (CMSException);
};

}

#endif
