/**
 *   File: logger.h
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#ifndef CIEMATCMSLOGGER_H
#define CIEMATCMSLOGGER_H

#include <string>
#include <stdlib.h>
#include <iostream>
#include "timegeneral.h"

using namespace std;

namespace CiematCMS {

/** Nombre de la variable de entorno que permite definir el nivel
  * de log por defecto.
  */
#define ENV_LEVEL "LOGGER_LEVEL"
/** Etiqueta para identificar el logger por defecto.
 */
#define DEF_LOGGER_NAME "__DEFAULT_LOGGER__"

/** Macros para que resulte cmodo el uso del DEFAULT_LOGGER
 *
 * Basta con usar NOMBRE-MACRO(mensaje) y crea un log por defecto
 * majo.
 * Se emplea el parmetro de definicion __DEBUG__ para poder controlar
 * desde el entorno de compilacin, si se desea la inclusin del cdigo
 * de lo logger o no, de manera que se gane (algo de) velocidad y se genere
 * un cdigo ms compacto.
 */

#ifdef __DEBUG__

#define DEFLOGTRACE(mens) Logger::getdefaultLogger().trace(mens)
#define DEFLOGDEBUG(mens) Logger::getdefaultLogger().debug(mens)
#define DEFLOGINFO(mens)  Logger::getdefaultLogger().info(mens)
#define DEFLOGWARN(mens)  Logger::getdefaultLogger().warn(mens)
#define DEFLOGERROR(mens) Logger::getdefaultLogger().error(mens)
#define DEFLOGFATAL(mens) Logger::getdefaultLogger().fatal(mens)

#else

#define DEFLOGTRACE(mens)
#define DEFLOGDEBUG(mens)
#define DEFLOGINFO(mens)
#define DEFLOGWARN(mens)
#define DEFLOGERROR(mens)
#define DEFLOGFATAL(mens)

#endif
  
/**
 * Mecanismo de log muy sencillo.
 * Debemos ampliarlo para que, mediante un mtodo "static", cualquier
 * componente/clase pueda recuperar una entidad tipo "Logger", registrada
 * en un repositorio "static" comn.
 * La eleccin del logger se har mediante un nombre identificador unico
 * En caso que no exista el logger, deber crease uno por defecto.
 * El constructor de "Logger" tendr que modificarse para que registre
 * cualquier instancia creada en el repositorio interno.
 *
 * @author J.M. <josemanuel.cela@ciemat.es>
 */
class Logger{

  public:
    Logger(string name);
    ~Logger();

    enum LOG_LEVELS {
      LOG_LEVEL_ALL,
      LOG_LEVEL_TRACE,
      LOG_LEVEL_DEBUG,
      LOG_LEVEL_INFO,
      LOG_LEVEL_WARN,
      LOG_LEVEL_ERROR,
      LOG_LEVEL_FATAL,
      LOG_LEVEL_OFF
    };

    // Permiten fijar o consultar el nivel de LOG prefijado.
    void setLevel(LOG_LEVELS level);
    LOG_LEVELS getLevel(void);
    // Activan o desactivan el mostrado de hora y nombre.
    void showTime(void);
    void hideTime(void);
    void showName(void);
    void hideName(void);

    // Aada un mensaje con el nivel correspondiente
    void trace(string mensaje);
    void debug(string mensaje);
    void info(string mensaje);
    void warn(string mensaje);
    void error(string mensaje);
    void fatal(string mensaje);

    // Devuelve un logger por nombre
    /* De momento no lo implementamos. Conviene crear antes un objeto
    para la gestin de listas doblemente enlazadas en el que se almacene
    la lista de "Logger" de forma cmoda */
    // static Logger &getLogger(string nombre);

    /* En espera que se cree el mecanismo de recuperacin de "loggers" por
    nombre, la funcin de devolucion del "default-logger" hace uso de una
    referencia interna tipo static. Cuando todo esto se haga bien,
    debera estar incluido en el contenedor de Loggers */
    static Logger &getdefaultLogger(void);

  protected:
    bool showLogName, showDateTime;
    LOG_LEVELS currentLogLevel;

    bool isLevelEnable (LOG_LEVELS level);
    void write (string cadena);
    void log(LOG_LEVELS nivel, string mensaje);

  private:
    string logName;
    TimeGeneral timeAuxiliar;

    static Logger defaultLogger;
};

}

#endif
