/**
 *   File: pca9564.h
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#ifndef CIEMATCMSPCA9564_H
#define CIEMATCMSPCA9564_H

#include "i_i2cmaster.h"
#include "i_vmeslave.h"
#include "cmsexception.h"
#include "timegeneral.h"
#include <unistd.h>

namespace CiematCMS {

/**
 * Contiene la lgica del integrado PCA9564.
 *
 * Slo se ha desarrollado como MASTER del bus I2C, pese
 * a que soporta la funcionalidad de actuar parcialmente como
 * esclavo.
 * Habr que hacer bastante trabajo para integrar, cuando se
 * necesite, esta otra funcionalidad.
 * 
 * @author J.M. <josemanuel.cela@ciemat.es>
 */

class PCA9564 : public I_I2CMaster, I_VMESlave {

  public:
    PCA9564(I_VMEMaster &master);
    ~PCA9564();

    typedef unsigned char SLAVEADDS[128];

    // Construye los mtodos definidos en el interfaz "I_I2CMaster"
    /* OJO!! En estos 2 mtodos el bit AA del registro de control
    se mantine a 0 para que *NO* se pueda activar el modo "slave" del
    dispositivo.
    Si se implementa la funcionalidad de "slave" deber revisarse este
    cdigo */
    void readI2CBytes(unsigned char slaveAdd, unsigned char* datos,
                      unsigned int bytesNumber)
        throw (CMSException);
    void writeI2CBytes(unsigned char slaveAdd, unsigned char* datos,
                       unsigned int bytesNumber)
        throw (CMSException);

    /* Utilidad para barrer todo el rango de direcciones de esclavo I2C
    vlidas (0x00 - 0x7F) y devolver un array con aquellas que han
    respondido correctamente al intento de acceso.
    Permite, as, detectar que esclavos hay conectados al bus.

    El array debe contener suficiente espacio para poder almacenar la
    mxima lista posible: 128 posiciones. 
    */
    void testI2CSlaves(SLAVEADDS lista, unsigned long demora=10000);

    void setControlRegItem (string value);
    string getControlRegItem (void);

    void setStatusRegItem (string value);
    string getStatusRegItem (void);

    void setDataRegItem(string value);
    string getDataRegItem(void);

    void setOwnAddRegItem (string value);
    string getOwnAddRegItem (void);

  private:
    /* Tiempo, en usecs, necesario para que se active el oscilador
    interno (mas un poquito de margen) */
    static int const OSC_START_TIME=700;
    // Timeout por si se queda pillado mientras espera al set de SI.
    static unsigned long const WAITSI_TIMEOUT=300000;
    /* Tiempo de espera de la activacin de SI entre ciclo y
    ciclo de pooling */
    static unsigned long const ESPERA_WAITSI=100;
    // Bits para leer o escribir.
    static unsigned char const WRITEBIT=0, READBIT=1;

    // Para comprobar el tiempo transcurrido y disparar TIMEOUTS.
    TimeGeneral tiempoAuxiliar;

    /* Mscaras para obtener los bits del registro de CONTROL */
    enum CONTROLBITSMASK {
      ASSERTACK= 0x80,
      ENABLESIO= 0x40,
      STARTFLAG= 0x20,
      STOPFLAG=  0x10,
      SERIALINT= 0x08,
      CLOCKR2=   0x04,
      CLOCKR1=   0x02,
      CLOCKR0=   0x01
    };

    /* Un gevo de cdigos de estado posibles depus de cada paso del
    protocolo */
    enum STATUS_VALUES {
      STSTARTOK=0x08,     // Start secuence OK
      STSTARTOKREP=0x10,  // Start secuence (REPEATED) OK
      STSLVADDRECREADOK=0x40, // Slave address recived OK. READ-MODE
      STSLVADDRECWRITOK=0x18, // Slave address recived OK. WRITE-MODE
      STWRITEOK=0x28,   // Envo correcto de byte al escribir.
      STSTOPOK=0xF8       // Stop OK.
    };

    /* Direcciones de registros en formato STRING para el empleo con HAL*/
    string  controlRegItem,
            statusRegItem,
            dataRegItem,
            ownAddRegItem;

    /* Flag para identificar si ya se ha activado el mecanismo de IO del
    integrado y evitar, as, inicializaciones redundantes innecesarias */
    bool sioStarted;
    // Slo deben emplearse los 3 bits MENOS significativos.
    unsigned char frecuency;

    // Mtodo que espera hasta que se active el bit SI.
    void waitSI (void) throw (CMSException);
};

}

#endif
