/**
 *   File: configfileparser.h
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#ifndef CIEMATCMSCONFIGFILEPARSER_H
#define CIEMATCMSCONFIGFILEPARSER_H

#include <fstream>
#include <string>
#include <map>
#include <algorithm>
#include <cctype>

#include "cmsexception.h"
#include "nstring.h"
#include "logger.h"
#include "configfilesection.h"
#include "i_configtree.h"

#define __LITERALES_INVALIDOS__ "#@$%&!?+-*/<>="

using namespace std;

namespace CiematCMS {

/**
 * Analiza un fichero de configuracion, en texto plano, con estructura
 *
 * ***************
 * # Comentario
 * 
 * var0=valor0   # Otro comentario
 * 
 * [Seccion 1]
 *   var1=valor1
 *   var2=valor2
 *
 * [Seccion 2]
 *   var3=valor3
 *   var2=valor2
 *
 * .....
 * .....
 * 
 * ***************
 *
 * 
 * Interpreta todo como "strings".
 * 
 * Variables con el mismo nombre pero en distinta seccin se tratan como
 * distintas.
 * Se pueden incluir variables sin seccin.
 *
 *	@author J.M. <josemanuel.cela@ciemat.es>
 */
class ConfigFileParser : public I_ConfigTree {

  public:
    ConfigFileParser(string fileName);
    ConfigFileParser();
    ~ConfigFileParser();

    /* Permite almacenar en la ruta absoluta ("path") especificada
    el fichero en cuestion.*/
    void saveFile(string path) throw (CMSException);

    /* Analiza el fichero. Si se produce excepcion, los cdigos emitidos
    son:
     - "-100": No se puede abrir el fichero o no existe
     - Nmero positivo: nmero de lnea donde se ha detectado el error */
    void parseFile(void) throw (CMSException);

    /* Tras el anlisis del fichero ("parseo") permite obtener el valor
    asociado a una variable dentro de una seccion determinada.
    Lanza una excepcin cuando: no existe la variable; o no existe
    la seccin; o no est "parseado" el fichero */
    string getValue(string section, string variable)
        throw (CMSException);

    /* Devuelve el valor de una variable sin seccion. Asociada a la
    seccin ROOT */
    string getValue(string variable) throw (CMSException);

    /* Permite modificar (en memoria) el valor de una variable
    dentro de una seccin */
    void setValue(string section, string variable, string value)
        throw (CMSException);

    /* Establece el fichero para parsear */
    void setInputFile(string filename) throw (CMSException);

    /* Recorta los espacios de una cadena por la derecha */
    string rightTrim (string value);

    /* Recorta espacios de una cadena por la izquierda */
    string leftTrim (string value);

    /* Para el interface I_ConfigTree */
    string getKeyValue(string key);

    /* Permiten establecer que el anlisis del fichero se realice
    teniendo o no en cuenta la diferencia entre MAYUS y MINUS.
    Esto afecta al nombre de las variable y las secciones. Pero no
    a los VALORES de las variables que figuran inalterados.
    Si se quiere que estos sean transformados a MAYUS o MINUS,
    se han de configurar con los mtodos "convert*" siguientes */
    void setCaseSensitive(void);
    void unsetCaseSensitive(void);

    /* Convierte o no el valor de las variables a MAYUS */
    void setVar2Upper(void);
    /* Convierte o no el valor de las variables a MINUS */
    void setVar2Lower(void);
    /* Deja el valor de las variables sin tocar MAYUS / MINUS */
    void setOwnCase(void);

    unsigned int getTotalSections(void);
    unsigned int getTotalRootVars(void);
    unsigned int getTotalVarInSec(string section);

  private:

    /* Almacn de secciones */
    map<string, ConfigFileSection> sectionMap;
    /* Almacn de variables sin seccin. Se localizan al comienzo del
    fichero*/
    map<string, string> rootVarMap;

    /* Nombre del fichero de entrada */
    string inputFilename;
    /* "stream" para el fichero de entrada */
    ifstream inputFile;
    /* Nmero de lineas en el fichero */
    unsigned int totalLines, totalSections;

    /* Identificadores para los diferentes tipos de lineas
    que puede haber. Las lineas en blanco corresponden tambien al tipo
    COMMENT */
    enum LINETYPE {
      COMMENT, SECTION, VARIABLE, ERROR, OK
    };

    enum CASESENS {CASE_SENS, NO_CASE_SENS};
    int caseSensStatus;

    enum CASECONVERSION {CONVERT_UPPER, CONVERT_LOWER, LEAVE_CASE};
    int caseConvert;

    /* Analiza una lnea hasta llegar a la marca de comentario "#".
    Devuelve un indicador del tipo de linea.
    Devuelve, tambin, por parmetros:
       - En "literal" el nombre de la seccin
       - En "value" el valor asociado (despus del smbolo "=") en el
         caso de una variable
    El tipo "ERROR" se devuelve al encontrar un error sintctico */
    int parseLine(string line, string &literal, string &value);

    /* Analiza uns cadena de forma que no contenga los caracteres
    prohibidos dentro de un nombre de seccin. Devuelve "ERROR" en caso
    de errores y "SECTION" en caso correcto */
    int validateLiteral(string literal);
    void addSection (string sectionName) throw (CMSException);
    void addSectionVar (string sectionName, string varName,
                        string varValue) throw (CMSException);

    void addRootVar(string varName, string varValue) throw (CMSException);

    string toUpper(string cadena);
    string toLower(string cadena);

    /* Determina si es necesario cambiar el "case de una cadena y
    lo realiza*/
    string convertCase(string cadena);

    /* Transforma la cadena a maysculas si *NO* hay que distinguir entre
    mayus y minus */
    string analizeCaseSens(string cadena);
};

}

#endif
