/**
 *   File: ds2482.cpp
 *
 *   Copyright (C) 2006 by J.M. 
 *   josemanuel.cela@ciemat.es
 */
#include "ds2482.h"
#include "timegeneral.h"

namespace CiematCMS {

//**********************************************************
//*********** CONSTRUCTORES Y DESTRUCTORES **************
//**********************************************************
DS2482::DS2482(I_I2CMaster &master)
  : I_OneWireMaster(), I_I2CSlave(master){
  DEFLOGINFO("Creamos un objeto DS2482");

  currentChannel= CH0;
}

DS2482::~DS2482() {
  DEFLOGINFO("Destruimos un objeto DS2482");
}

//**********************************************************
//*********** MTODOS PBLICOS **************
//**********************************************************
/** Lee un bit del bus 1-Wire.
 *
 * Este mtodo y su complementario, son tiles para operaciones de
 * acceso a ROM de los esclavos del bus, en las que se requiere el
 * envio y lectura de bits individuales.
 */
bool DS2482::singleBit1WRead(void) throw (CMSException){
  return singleBitOperation(true);
}

/** Escribe un bit en el bus 1-Wire
 *
 * En este integrado, este mtodo y el anterior comparten la misma lgica
 * salvo detalles, por lo que pese a que se desdoblan en 2 en la interfaz
 * pblica de la clase, internamente invocan al mismo mtodo
 * (privado) cambiando un par de parmetros.
 */
void DS2482::singleBit1WWrite(bool bitValue) throw (CMSException){
  singleBitOperation(bitValue);
}

/**
 * Lee un byte a travs del canal seleccionado por defecto.
 * Debe controlar el protocolo y esperar a que la actividad
 * en el bus haya finalizado (bit 1WB == 0) antes de devolver
 * el control.
 */
unsigned char DS2482::read1WByte (void) throw (CMSException){
  DEFLOGINFO("DS2482: Entramos en read1WByte");

  // Array con el comando READ_BYTE.
  unsigned char read1wCmd[1]= {char(READ_BYTE)};

  // Ejecutamos el comando "leer"
  masterI2C.writeI2CBytes(slaveI2COwnAddress, read1wCmd, 1);
  waitUntil_1W_Stop();

  DEFLOGDEBUG("DS2482: Enviado comando READ_BYTE");

  // Fijamos el registro de datos para leerlo. Usamos un array
  // de 2 bytes: COMANDO + Registro-elegido
  unsigned char selectDataRegCmd[2]={char(SET_READPOINTER) , '\0'};
  selectDataRegCmd[1]=char(DATAREG);
  masterI2C.writeI2CBytes(slaveI2COwnAddress, selectDataRegCmd, 2);

  DEFLOGDEBUG("DS2482: Enviado comando SET_READPOINTER");

  // Leemos el dato
  unsigned char dataByte[1]={'\0'};
  masterI2C.readI2CBytes(slaveI2COwnAddress, dataByte, 1);

  DEFLOGINFO("DS2482: Salimos de read1WByte");

  return dataByte[0];
};

void DS2482::write1WByte (unsigned char byte) throw (CMSException){

  DEFLOGINFO("DS2482: Entramos en write1WByte");

  // Array con el comando WRITE_BYTE y con espacio para el byte
  // que se va a escribir.
  unsigned char write1wCmd[2]= {char(WRITE_BYTE), '\0'};
  write1wCmd[1]=byte;

  masterI2C.writeI2CBytes(slaveI2COwnAddress, write1wCmd, 2);
  waitUntil_1W_Stop();

  DEFLOGDEBUG("DS2482: Enviado comando WRITE_BYTE");

  DEFLOGINFO("DS2482: Entramos en write1WByte");
};

void DS2482::reset1W (void) throw (CMSException){
  // Array con el comando RESET_1W.
  unsigned char reset1wCmd[1]= {char(RESET_1W)};

  masterI2C.writeI2CBytes(slaveI2COwnAddress, reset1wCmd, 1);
  waitUntil_1W_Stop();

  DEFLOGDEBUG("DS2482: Enviado comando RESET_1W");
};

//**********************************************************
//*********** MTODOS GET Y SET **************
//**********************************************************
unsigned DS2482::getChannel() const {
    return currentChannel;
    // Convendra meter cdigo para verificar qu canal es
    // el que el integrado "cree" tener activo.
}

void DS2482::setChannel(enum CHANNEL_CODES dato)
    throw (CMSException) {

  DEFLOGINFO("DS2482: Entramos en setChannel");

  // Array para almacenar la orden de cambio de canal.
  unsigned char selChannelCmd[2]= {char(SELECT_CHANNEL), '\0'};
  // Para verificar el canal seleccionado
  unsigned char chVerif[1];

  selChannelCmd[1]=dato;

  // Fijamos el canal y verificamos que se corresponde con el
  // elegido. En caso contrario, mandamos una excepcin.
  masterI2C.writeI2CBytes(slaveI2COwnAddress, selChannelCmd, 2);
  masterI2C.readI2CBytes(slaveI2COwnAddress, chVerif, 1);

  if ((dato == CH0 && chVerif[0] != CH0_RESP) ||
      (dato == CH1 && chVerif[0] != CH1_RESP) ||
      (dato == CH2 && chVerif[0] != CH2_RESP) ||
      (dato == CH3 && chVerif[0] != CH3_RESP) ||
      (dato == CH4 && chVerif[0] != CH4_RESP) ||
      (dato == CH5 && chVerif[0] != CH5_RESP) ||
      (dato == CH6 && chVerif[0] != CH6_RESP) ||
      (dato == CH7 && chVerif[0] != CH7_RESP))
  {
        // Preparamos el mensaje para la excepcin.
        string mensajeExcp= "No se puede fijar el canal elegido (";
        char canalTXTTemp[30];
        sprintf (canalTXTTemp, "%#X", dato);
        mensajeExcp+= canalTXTTemp;
        mensajeExcp+= "). Respuesta: ";
        sprintf (canalTXTTemp, "%#X", chVerif[0]);
        mensajeExcp+= canalTXTTemp;

        throw (CMSException("DS2482::setCurrentChannel", mensajeExcp, 2));
  }
  else currentChannel = dato;

  DEFLOGINFO("DS2482: Salimos de setChannel");
}

//**********************************************************
//*********** MTODOS PRIVADOS **************
//**********************************************************
/*!
 * \fn CiematCMS::DS2482::waitUntil_1W_Stop (void)
 * Aguarda hasta que el bit 1WB se fija a LOW. Mientras est
 * HIGH indica que el bus 1-W est ocupado transmitiendo informacin.
 * Se realizan ciclos de polling en cada uno de los cuales
 * se comprueba el valor del bit.
 * Entre ciclos se espera un tiempo prefijado.
 */
void DS2482::waitUntil_1W_Stop (void) throw (CMSException){

  DEFLOGINFO("DS2482: Entramos en waitUntil_1W_Stop");

  // Vector para almacenar el registro de estado.
  unsigned char status[]={0};
  // Vector con el comando para apuntar al registro de estado
  unsigned char pointToStatusCmd[2]= {
    char(SET_READPOINTER),char(STATUSREG)
  };

  // Para obtener la hora actual y operar con microsegundos con
  // comodidad.
  struct timeval tinicial;
  tiempoAuxiliar.getActualTime(&tinicial);

  /* Fijamos el puntero de lectura al registro de STATUS.
  Colocamos aqu esta lnea, en vez de dentro del bucle, para ahorrar
  accesos I2C y acelerar un poco el proceso */
  masterI2C.writeI2CBytes(slaveI2COwnAddress, pointToStatusCmd, 2);

  do {
    // Hamos polling del registro de estado.
    masterI2C.readI2CBytes(slaveI2COwnAddress, status ,1);
    usleep (WAITTIME1WBUSY);

    // Abortamos el bucle en caso de demasiada tardanza, con 
    // una bonita excepcion.
    if (tiempoAuxiliar.utimeElapsed(&tinicial) > TIMEOUT)
      throw (
          CMSException("DS2482::waitUntil_1W_Stop", "Timeout", 1000)
          );

  } while ((status[0] & WIREBUSY)); // Repetimos hasta que libere el BUSY.

  DEFLOGINFO("DS2482: Salimos de waitUntil_1W_Stop");
}

/** Implementa el comando "1-Wire single bit" vlido tanto para leer
 * como para escribir 1 bit sobre el bus.
 *
 * La lectura se realiza *escribiendo* primero un 1 en el bus, siendo
 * el esclavo el que posteriormente lleva a cabo la escritura del dato
 * en el mismo.
 * El integrado resuelve el protocolo de forma transparente, dejando el
 * resultado de la lectura en el bit de status SBR (single-bit-result).
*/
bool DS2482::singleBitOperation(bool bitValue) throw (CMSException){

  DEFLOGINFO("DS2482: Entramos en singleBitOperation");

  // Array con el comando SINGLEBIT_OPER y con espacio para el byte_bit
  // que se va a escribir.
  unsigned char singleBitOpCmd[2]= {char(SINGLEBIT_OPER), '\0'};

  /* Escribimos 1  0 segn corresponda.
  Siguiendo las indicaciones del datasheet, el bit ms significativo
  es el que debe contener el valor que se quiere escribir (bit-7) */
  if (bitValue) singleBitOpCmd[1]=0x80;
  else singleBitOpCmd[1]=0x00;

  masterI2C.writeI2CBytes(slaveI2COwnAddress, singleBitOpCmd, 2);
  waitUntil_1W_Stop();

  DEFLOGDEBUG("DS2482: Enviado comando SINGLEBIT_OPER");

  /* Leemos el registro de estado para capturar el valor del bit
  single-bit-result */
  // Vector para almacenar el registro de estado.
  unsigned char status[]={0};
  // Vector con el comando para apuntar al registro de estado
  unsigned char pointToStatusCmd[2]= {
    char(SET_READPOINTER),char(STATUSREG)
  };
  masterI2C.writeI2CBytes(slaveI2COwnAddress, pointToStatusCmd, 2);
  masterI2C.readI2CBytes(slaveI2COwnAddress, status, 1);

  DEFLOGINFO("DS2482: Salimos de singleBitOperation");

  if (status[0] & SINGELBITRESULT) return true;
  else return false;
}

}
